<?php

namespace Deal\Deal\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Deal\Deal\Http\Requests\ShopRequest;
use Deal\Deal\Interfaces\ShopRepositoryInterface;
use Deal\Deal\Models\Shop;

/**
 * Resource controller class for shop.
 */
class ShopResourceController extends BaseController
{

    /**
     * Initialize shop resource controller.
     *
     * @param type ShopRepositoryInterface $shop
     *
     * @return null
     */
    public function __construct(ShopRepositoryInterface $shop)
    {
        parent::__construct();
        $this->repository = $shop;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Deal\Deal\Repositories\Criteria\ShopResourceCriteria::class);
    }

    /**
     * Display a list of shop.
     *
     * @return Response
     */
    public function index(ShopRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Deal\Deal\Repositories\Presenter\ShopPresenter::class)
                ->$function();
        }

        $shops = $this->repository->paginate();

        return $this->response->title(trans('deal::shop.names'))
            ->view('deal::shop.index', true)
            ->data(compact('shops'))
            ->output();
    }

    /**
     * Display shop.
     *
     * @param Request $request
     * @param Model   $shop
     *
     * @return Response
     */
    public function show(ShopRequest $request, Shop $shop)
    {

        if ($shop->exists) {
            $view = 'deal::shop.show';
        } else {
            $view = 'deal::shop.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('deal::shop.name'))
            ->data(compact('shop'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new shop.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ShopRequest $request)
    {

        $shop = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('deal::shop.name')) 
            ->view('deal::shop.create', true) 
            ->data(compact('shop'))
            ->output();
    }

    /**
     * Create new shop.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ShopRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $shop                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('deal::shop.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('deal/shop/' . $shop->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/deal/shop'))
                ->redirect();
        }

    }

    /**
     * Show shop for editing.
     *
     * @param Request $request
     * @param Model   $shop
     *
     * @return Response
     */
    public function edit(ShopRequest $request, Shop $shop)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('deal::shop.name'))
            ->view('deal::shop.edit', true)
            ->data(compact('shop'))
            ->output();
    }

    /**
     * Update the shop.
     *
     * @param Request $request
     * @param Model   $shop
     *
     * @return Response
     */
    public function update(ShopRequest $request, Shop $shop)
    {
        try {
            $attributes = $request->all();

            $shop->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('deal::shop.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('deal/shop/' . $shop->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('deal/shop/' . $shop->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the shop.
     *
     * @param Model   $shop
     *
     * @return Response
     */
    public function destroy(ShopRequest $request, Shop $shop)
    {
        try {

            $shop->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('deal::shop.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('deal/shop/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('deal/shop/' . $shop->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple shop.
     *
     * @param Model   $shop
     *
     * @return Response
     */
    public function delete(ShopRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('deal::shop.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('deal/shop'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/deal/shop'))
                ->redirect();
        }

    }

    /**
     * Restore deleted shops.
     *
     * @param Model   $shop
     *
     * @return Response
     */
    public function restore(ShopRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('deal::shop.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/deal/shop'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/deal/shop/'))
                ->redirect();
        }

    }

}
